@@ -1,3 +1,14 @@
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright  2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
+
 /*- This is a -*- C -*- compatible code file
  *
  * Code for SUNOS5_INSTRUMENTATION
@@ -114,7 +125,7 @@ mibmap          Mibmap[MIBCACHE_SIZE] = {
     {0},
 };
 
-static int      sd = -1;        /* /dev/ip stream descriptor. */
+static int      sd = -2;        /* /dev/ip stream descriptor. */
 
 /*-
  * Static function prototypes (use void as argument type if there are none)
@@ -227,7 +238,7 @@ getKstatInt(const char *classname, const char *statname,
     if (kstat_fd == 0) {
 	kstat_fd = kstat_open();
 	if (kstat_fd == 0) {
-	    snmp_log(LOG_ERR, "kstat_open(): failed\n");
+	    snmp_log_perror("kstat_open");
 	}
     }
     if ((ksc = kstat_fd) == NULL) {
@@ -297,6 +308,7 @@ getKstat(const char *statname, const char *varname, void *value)
     int             ret;
     u_longlong_t    val;    /* The largest value */
     void           *v;
+    static char    buf[128];
 
     if (value == NULL) {      /* Pretty useless but ... */
 	v = (void *) &val;
@@ -307,7 +319,7 @@ getKstat(const char *statname, const char *varname, void *value)
     if (kstat_fd == 0) {
 	kstat_fd = kstat_open();
 	if (kstat_fd == 0) {
-	    snmp_log(LOG_ERR, "kstat_open(): failed\n");
+	    snmp_log_perror("kstat_open");
 	}
     }
     if ((ksc = kstat_fd) == NULL) {
@@ -383,8 +395,10 @@ getKstat(const char *statname, const char *varname, void *value)
 	if (strcmp(d->name, varname) == 0) {
 	    switch (d->data_type) {
 	    case KSTAT_DATA_CHAR:
-		*(char *)v = (int)d->value.c;
-		DEBUGMSGTL(("kernel_sunos5", "value: %d\n", (int)d->value.c));
+		DEBUGMSGTL(("kernel_sunos5", "value: %s\n", d->value.c));
+		*(char **)v = buf;
+		buf[sizeof(buf)-1] = 0;
+		strncpy(buf, d->value.c, sizeof(buf)-1);
 		break;
 #ifdef KSTAT_DATA_INT32         /* Solaris 2.6 and up */
 	    case KSTAT_DATA_INT32:
@@ -461,7 +475,7 @@ getKstatString(const char *statname, const char *varname,
     if (kstat_fd == 0) {
         kstat_fd = kstat_open();
         if (kstat_fd == 0) {
-            snmp_log(LOG_ERR, "kstat_open(): failed\n");
+            snmp_log_perror("kstat_open");
         }
     }
     if ((ksc = kstat_fd) == NULL) {
@@ -538,7 +552,7 @@ getKstatString(const char *statname, const char *varname,
             switch (d->data_type) {
             case KSTAT_DATA_CHAR:
                 value[value_len-1] = '\0';
-                strncpy(value, value_len-1, d->value.c); 
+                strncpy(value, d->value.c, value_len-1); 
                 DEBUGMSGTL(("kernel_sunos5", "value: %s\n", d->value.c));
                 break;
             default:
@@ -793,25 +807,33 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size,
      * Open the stream driver and push all MIB-related modules 
      */
 
-    if (sd == -1) {         /* First time */
+    if (sd == -2) {         /* First time */
 	if ((sd = open(STREAM_DEV, O_RDWR)) == -1) {
+	    snmp_log_perror(STREAM_DEV);
 	    ret = -1;
 	    goto Return;
 	}
 	if (ioctl(sd, I_PUSH, "arp") == -1) {
+	    snmp_log_perror("I_PUSH arp");
 	    ret = -1;
 	    goto Return;
 	}
 	if (ioctl(sd, I_PUSH, "tcp") == -1) {
+	    snmp_log_perror("I_PUSH tcp");
 	    ret = -1;
 	    goto Return;
 	}
 	if (ioctl(sd, I_PUSH, "udp") == -1) {
+	    snmp_log_perror("I_PUSH udp");
 	    ret = -1;
 	    goto Return;
 	}
 	DEBUGMSGTL(("kernel_sunos5", "...... modules pushed OK\n"));
     }
+    if (sd == -1) {
+	ret = -1;
+	goto Return;
+    }
 
     /*
      * First, use bigger buffer, to accelerate skipping unwanted messages
@@ -929,7 +951,7 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size,
 	}
     }
  Return:
-    ioctl(sd, I_FLUSH, FLUSHRW);
+    if (sd >= 0) ioctl(sd, I_FLUSH, FLUSHRW);
     DEBUGMSGTL(("kernel_sunos5", "...... getmib returns %d\n", ret));
     return ret;
 }
@@ -1123,13 +1145,13 @@ getif(mib2_ifEntry_t *ifbuf, size_t size, req_e req_type,
 		goto Return;
 	    }
 
-	    if (getKstatInt(NULL,ifrp->ifr_name, "brdcstrcv",&ifp->ifInNUcastPkts)==0&&
-		getKstatInt(NULL,ifrp->ifr_name, "multircv", &l_tmp) == 0) {
+	    if (getKstatInt(NULL,ifrp->ifr_name, "brdcstrcv",&ifp->ifInNUcastPkts)==1&&
+		getKstatInt(NULL,ifrp->ifr_name, "multircv", &l_tmp) == 1) {
 		ifp->ifInNUcastPkts += l_tmp;
 	    }
 
-	    if (getKstatInt(NULL,ifrp->ifr_name,"brdcstxmt",&ifp->ifOutNUcastPkts)==0&&
-		getKstatInt(NULL,ifrp->ifr_name, "multixmt", &l_tmp) == 0) {
+	    if (getKstatInt(NULL,ifrp->ifr_name,"brdcstxmt",&ifp->ifOutNUcastPkts)==1&&
+		getKstatInt(NULL,ifrp->ifr_name, "multixmt", &l_tmp) == 1) {
 		ifp->ifOutNUcastPkts += l_tmp;
 	    }
 	}
