@@ -3,6 +3,17 @@
  *
  */
 
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright  2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
+
 #include <net-snmp/net-snmp-config.h>
 #include "mibII_common.h"
 
@@ -181,7 +192,7 @@ udpTable_handler(netsnmp_mib_handler          *handler,
 #endif
                 break;
             case UDPLOCALPORT:
-                port = ntohs(entry->UDPTABLE_LOCALPORT);
+                port = ntohs((u_short)entry->UDPTABLE_LOCALPORT);
 	        snmp_set_var_typed_value(requestvb, ASN_INTEGER,
                                  (u_char *)&port, sizeof(port));
                 break;
@@ -256,7 +267,7 @@ udpTable_next_entry( void **loop_context,
      */
     snmp_set_var_value(index, (u_char *)&udp_head[i].UDPTABLE_LOCALADDRESS,
                                   sizeof(udp_head[i].UDPTABLE_LOCALADDRESS));
-    port = ntohs(udp_head[i].UDPTABLE_LOCALPORT);
+    port = ntohs((u_short)udp_head[i].UDPTABLE_LOCALPORT);
     snmp_set_var_value(index->next_variable,
                                (u_char*)&port, sizeof(port));
     /*
@@ -271,8 +282,15 @@ udpTable_next_entry( void **loop_context,
 void
 udpTable_free(netsnmp_cache *cache, void *magic)
 {
+#ifdef WIN32
+    if (udp_head) {
+		/* the allocated structure is a count followed by table entries */
+		free((char *)(udp_head) - sizeof(DWORD));
+	}
+#else
     if (udp_head)
         free(udp_head);
+#endif
     udp_head = NULL;
     udp_size = 0;
 }
@@ -540,11 +558,13 @@ udpTable_load(netsnmp_cache *cache, void *vmagic)
     }
     if (status == NO_ERROR) {
         DEBUGMSGTL(("mibII/udpTable", "Loaded UDP Table\n"));
-        udp_size = pUdpTable->dwNumEntries;
+        udp_size = pUdpTable->dwNumEntries -1;  /* entries are counted starting with 0 */
         udp_head = pUdpTable->table;
         return 0;
     }
     DEBUGMSGTL(("mibII/udpTable", "Failed to load UDP Table (win32)\n"));
+    if (pUdpTable)
+	free(pUdpTable);
     return -1;
 }
 #else                           /* WIN32 */
