@@ -1,6 +1,16 @@
 /*
  * SNMPv3 View-based Access Control Model
  */
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright  2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
 
 #include <net-snmp/net-snmp-config.h>
 
@@ -1649,6 +1659,10 @@ write_vacmSecurityToGroupStatus(int action,
                 long_ret = RS_NOTREADY;
                 return SNMP_ERR_INCONSISTENTVALUE;
             }
+            if (long_ret == RS_DESTROY && geptr->storageType == ST_PERMANENT) {
+                free(newName);
+                return SNMP_ERR_WRONGVALUE;
+            }
         } else {
             if (long_ret == RS_ACTIVE || long_ret == RS_NOTINSERVICE) {
                 free(newName);
@@ -1928,6 +1942,11 @@ write_vacmAccessStatus(int action,
                 free(newContextPrefix);
                 return SNMP_ERR_INCONSISTENTVALUE;
             }
+            if (long_ret == RS_DESTROY && aptr->storageType == ST_PERMANENT) {
+                free(newGroupName);
+                free(newContextPrefix);
+                return SNMP_ERR_WRONGVALUE;
+            }
         } else {
             if (long_ret == RS_ACTIVE || long_ret == RS_NOTINSERVICE) {
                 free(newGroupName);
@@ -2415,6 +2434,11 @@ write_vacmViewStatus(int action,
                 long_ret = RS_NOTREADY;
                 return SNMP_ERR_INCONSISTENTVALUE;
             }
+            if (long_ret == RS_DESTROY && vptr->viewStorageType == ST_PERMANENT) {
+                free(newViewName);
+                free(newViewSubtree);
+                return SNMP_ERR_WRONGVALUE;
+            }
         } else {
             if (long_ret == RS_ACTIVE || long_ret == RS_NOTINSERVICE) {
                 free(newViewName);
