@@ -3,6 +3,10 @@
  *
  *
  */
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
 /***********************************************************
 	Copyright 1988, 1989 by Carnegie Mellon University
 	Copyright 1989	TGV, Incorporated
@@ -25,6 +29,13 @@ USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************/
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright  2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
+
 /*
  * additions, fixes and enhancements for Linux by Erik Schoenfelder
  * (schoenfr@ibr.cs.tu-bs.de) 1994/1995.
@@ -714,8 +725,10 @@ var_ipRouteEntry(struct variable * vp,
     if (*length == IP_ROUTENAME_LENGTH) /* Assume that the input name is the lowest */
         memcpy((char *) lowest, (char *) name,
                IP_ROUTENAME_LENGTH * sizeof(oid));
-    else
+    else {
         name[IP_ROUTEADDR_OFF] = (oid) - 1;     /* Grhhh: to prevent accidental comparison :-( */
+	lowest[0] = 0xff;
+    }
     for (Nextentry.ipRouteDest = (u_long) - 2, req_type = GET_FIRST;;
          Nextentry = entry, req_type = GET_NEXT) {
         if (getMibstat(MIB_IP_ROUTE, &entry, sizeof(mib2_ipRouteEntry_t),
@@ -766,8 +779,9 @@ var_ipRouteEntry(struct variable * vp,
 
     switch (vp->magic) {
     case IPROUTEDEST:
-        long_return = Lowentry.ipRouteDest;
-        return (u_char *) & long_return;
+	*var_len = sizeof(uint32_t);
+        ipaddr_return = Lowentry.ipRouteDest;
+        return (u_char *) & ipaddr_return;
     case IPROUTEIFINDEX:
         long_return =
             Interface_Index_By_Name(Lowentry.ipRouteIfIndex.o_bytes,
@@ -786,8 +800,9 @@ var_ipRouteEntry(struct variable * vp,
         long_return = Lowentry.ipRouteMetric4;
         return (u_char *) & long_return;
     case IPROUTENEXTHOP:
-        long_return = Lowentry.ipRouteNextHop;
-        return (u_char *) & long_return;
+	*var_len = sizeof(uint32_t);
+        ipaddr_return = Lowentry.ipRouteNextHop;
+        return (u_char *) & ipaddr_return;
     case IPROUTETYPE:
         long_return = Lowentry.ipRouteType;
         return (u_char *) & long_return;
@@ -800,8 +815,9 @@ var_ipRouteEntry(struct variable * vp,
         long_return = Lowentry.ipRouteAge;
         return (u_char *) & long_return;
     case IPROUTEMASK:
-        long_return = Lowentry.ipRouteMask;
-        return (u_char *) & long_return;
+	*var_len = sizeof(uint32_t);
+        ipaddr_return = Lowentry.ipRouteMask;
+        return (u_char *) & ipaddr_return;
     default:
         DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipRouteEntry\n",
                     vp->magic));
