@@ -74,7 +74,7 @@ snmptrapd_parse_traphandle(const char *token, char *line)
 {
     char            buf[STRINGMAX];
     oid             obuf[MAX_OID_LEN];
-    int             olen = MAX_OID_LEN;
+    size_t          olen = MAX_OID_LEN;
     char           *cptr;
     netsnmp_trapd_handler *traph;
 
@@ -110,7 +110,7 @@ parse_forward(const char *token, char *line)
 {
     char            buf[STRINGMAX];
     oid             obuf[MAX_OID_LEN];
-    int             olen = MAX_OID_LEN;
+    size_t          olen = MAX_OID_LEN;
     char           *cptr;
     netsnmp_trapd_handler *traph;
 
@@ -124,7 +124,7 @@ parse_forward(const char *token, char *line)
         if (!read_objid(buf, obuf, &olen)) {
             char            buf1[STRINGMAX];
             snprintf(buf1,  sizeof(buf1),
-                    "Bad trap OID in traphandle directive: %s", buf);
+                    "Bad trap OID in forward directive: %s", buf);
             buf1[ sizeof(buf1)-1 ] = 0;
             config_perror(buf1);
             return;
@@ -592,7 +592,7 @@ int   print_handler(   netsnmp_pdu           *pdu,
 	    }
 	} else {
             if (print_format2) {
-                DEBUGMSGTL(( "snmptrapd", "print_format v1 = '%s'\n", print_format2));
+                DEBUGMSGTL(( "snmptrapd", "print_format v2 = '%s'\n", print_format2));
                 trunc = !realloc_format_trap(&rbuf, &r_len, &o_len, 1,
                                              print_format2, pdu, transport);
 	    } else {
@@ -758,8 +758,7 @@ do_external(char *cmd, struct hostent *host,
     snmp_set_quick_print(oldquick);
 }
 
-
-#define EXECUTE_FORMAT	"%b\n%B\n%V\n%v\n"
+#define EXECUTE_FORMAT	"%B\n%b\n%V\n%v\n"
 
 int   command_handler( netsnmp_pdu           *pdu,
                        netsnmp_transport     *transport,
@@ -772,6 +771,11 @@ int   command_handler( netsnmp_pdu           *pdu,
     DEBUGMSGTL(( "snmptrapd", "command_handler\n"));
     DEBUGMSGTL(( "snmptrapd", "token = '%s'\n", handler->token));
     if (handler && handler->token && *handler->token) {
+	netsnmp_pdu    *v2_pdu = NULL;
+	if (pdu->command == SNMP_MSG_TRAP)
+	    v2_pdu = convert_v1pdu_to_v2(pdu);
+	else
+	    v2_pdu = pdu;
         oldquick = snmp_get_quick_print();
         snmp_set_quick_print(1);
 
@@ -789,14 +793,14 @@ int   command_handler( netsnmp_pdu           *pdu,
          */
         if (handler && handler->format && *handler->format) {
             DEBUGMSGTL(( "snmptrapd", "format = '%s'\n", handler->format));
-            !realloc_format_trap(&rbuf, &r_len, &o_len, 1,
+            realloc_format_trap(&rbuf, &r_len, &o_len, 1,
                                              handler->format,
-                                             pdu, transport);
+                                             v2_pdu, transport);
         } else {
             DEBUGMSGTL(( "snmptrapd", "execute format\n"));
-            !realloc_format_trap(&rbuf, &r_len, &o_len, 1,
+            realloc_format_trap(&rbuf, &r_len, &o_len, 1,
                                              EXECUTE_FORMAT,
-                                             pdu, transport);
+                                             v2_pdu, transport);
 	}
 
         /*
@@ -804,6 +808,8 @@ int   command_handler( netsnmp_pdu           *pdu,
          */
         run_exec_command(handler->token, rbuf, NULL, 0);   /* Not interested in output */
         snmp_set_quick_print(oldquick);
+	if (pdu->command == SNMP_MSG_TRAP)
+	    snmp_free_pdu(v2_pdu);
     }
     return NETSNMPTRAPD_HANDLER_OK;
 }
@@ -837,11 +843,18 @@ int   forward_handler( netsnmp_pdu           *pdu,
 {
     netsnmp_session session, *ss;
     netsnmp_pdu *pdu2;
+    char buf[BUFSIZ], *cp;
 
     DEBUGMSGTL(( "snmptrapd", "forward_handler (%s)\n", handler->token));
 
     snmp_sess_init( &session );
-    session.peername = handler->token;
+    if (strchr( handler->token, ':') == NULL) {
+        snprintf( buf, BUFSIZ, "%s:%d", handler->token, SNMP_TRAP_PORT);
+        cp = buf;
+    } else {
+        cp = handler->token;
+    }
+    session.peername = cp;
     session.version  = pdu->version;
     ss = snmp_open( &session );
 
