@@ -1,5 +1,9 @@
 #include "generic.h"
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>	/* uint32_t */
+#endif
+
 #undef bsdlike
 #undef MBSTAT_SYMBOL
 #undef TOTAL_MEMORY_SYMBOL
@@ -7,6 +11,30 @@
 #undef HAVE_SOCKET
 #undef HAVE_SIGNAL
 
+/* Define if you have the gettimeofday function.  */
+/* Only when compiling Perl module                */
+#ifdef MINGW_PERL
+#define HAVE_GETTIMEOFDAY 1
+#endif
+
+/* Define if you have the gethostbyname function.  */
+#define HAVE_GETHOSTBYNAME 1
+
+/* Define if you have the gethostname function.  */
+#define HAVE_GETHOSTNAME 1
+
+/* Define if you have the getservbyname function.  */
+#define HAVE_GETSERVBYNAME 1
+
+/* Define if you have raise() instead of alarm() */
+#define HAVE_RAISE 1
+
+/* Define to 1 if you have the `execv' function. */
+#undef HAVE_EXECV
+
+/* Define to 1 if you have the `fork' function. */
+#undef HAVE_FORK
+
 /*
  * I'm sure there is a cleaner way to do this.
  * Probably should be in net_snmp_config.h and
@@ -19,16 +47,6 @@
 /* This was taken from the win32 config file. */
 #define EADDRINUSE		WSAEADDRINUSE
 
-/*
- * Not sure if the checking is required but cygwin
- * does it this way.
- */
-#if defined(__INT_MAX__) && __INT_MAX__ == 2147483647
-typedef int ssize_t;
-#else
-typedef long ssize_t;
-#endif
-
 /*
  * File io stuff. Odd that this is not defined by MinGW.
  * Maybe there is an M$ish way to do it.
@@ -46,4 +64,5 @@ typedef long ssize_t;
 struct timezone {
 	int tz_minuteswest;
 	int tz_dsttime;
-}
+};
+
