@@ -1,3 +1,14 @@
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
+
 #include <net-snmp/net-snmp-config.h>
 
 #include <stdio.h>
@@ -366,10 +377,10 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
     addr->sin_addr.s_addr = htonl(INADDR_ANY);
     addr->sin_family = AF_INET;
     if (remote_port > 0) {
-        addr->sin_port = htons(remote_port);
+        addr->sin_port = htons((u_short)remote_port);
     } else if (netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
 				  NETSNMP_DS_LIB_DEFAULT_PORT) > 0) {
-        addr->sin_port = htons(netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
+        addr->sin_port = htons((u_short)netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
 						 NETSNMP_DS_LIB_DEFAULT_PORT));
     } else {
         addr->sin_port = htons(SNMP_PORT);
@@ -396,7 +407,7 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
             if (atoi(cp) != 0) {
                 DEBUGMSGTL(("netsnmp_sockaddr_in",
                             "port number suffix :%d\n", atoi(cp)));
-                addr->sin_port = htons(atoi(cp));
+                addr->sin_port = htons((u_short)atoi(cp));
             }
         }
 
@@ -407,7 +418,7 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
              */
             DEBUGMSGTL(("netsnmp_sockaddr_in", "totally numeric: %d\n",
                         atoi(peername)));
-            addr->sin_port = htons(atoi(peername));
+            addr->sin_port = htons((u_short)atoi(peername));
         } else if (inet_addr(peername) != INADDR_NONE) {
             /*
              * It looks like an IP address.  
@@ -481,7 +492,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
     char           *cp = NULL;
     const char     *strmask = NULL;
     com2SecEntry   *e = NULL;
-    unsigned long   network = 0, mask = 0;
+    in_addr_t   network = 0, mask = 0;
 
     /*
      * Get security, source address/netmask and community strings.  
@@ -546,7 +557,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
          */
         network = inet_addr(source);
 
-        if (network == (unsigned long) -1) {
+        if (network == (in_addr_t) -1) {
             /*
              * Nope, wasn't a dotted quad.  Must be a hostname.  
              */
@@ -560,7 +571,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
                     config_perror("no IP address for source hostname");
                     return;
                 }
-                network = *((unsigned long *) hp->h_addr);
+                network = *((in_addr_t *) hp->h_addr);
             }
 #else                           /*HAVE_GETHOSTBYNAME */
             /*
@@ -587,7 +598,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
              * Try to interpret mask as a dotted quad.  
              */
             mask = inet_addr(strmask);
-            if (mask == (unsigned long) -1 &&
+            if (mask == (in_addr_t) -1 &&
                 strncmp(strmask, "255.255.255.255", 15) != 0) {
                 config_perror("bad mask");
                 return;
@@ -768,9 +779,10 @@ netsnmp_udp_create_ostring(const u_char * o, size_t o_len, int local)
     struct sockaddr_in addr;
 
     if (o_len == 6) {
+        unsigned short porttmp = (o[4] << 8) + o[5];
         addr.sin_family = AF_INET;
         memcpy((u_char *) & (addr.sin_addr.s_addr), o, 4);
-        addr.sin_port = ntohs((o[4] << 8) + o[5]);
+        addr.sin_port = porttmp;
         return netsnmp_udp_transport(&addr, local);
     }
     return NULL;
